Skuld.Comp.UncaughtThrow exception (skuld v0.2.3)

Exception raised when an Elixir throw is not caught within a Skuld computation.

This wraps the thrown value so it can be re-raised with the original stacktrace.