Skuld.Comp.InvalidComputation exception (skuld v0.1.8)

Raised when a non-computation value is used where a computation is expected.

This typically indicates a programming bug such as forgetting return(value) at the end of a comp block.