Skuld.Comp.UncaughtExit exception (skuld v0.2.3)
View SourceException raised when an Elixir exit is not caught within a Skuld computation.
This wraps the exit reason so it can be re-raised with the original stacktrace.
Exception raised when an Elixir exit is not caught within a Skuld computation.
This wraps the exit reason so it can be re-raised with the original stacktrace.